@@ -4,7 +4,7 @@
  * Copyright (C) 2012 Eric Wolf
  * This module is free software; it can be used under the terms of the artistic license
  *
- * Version 0.021
+ * Version 0.022
  */
 $.yote.util = {
     ids:0,
@@ -105,7 +105,10 @@ $.yote.util = {
 		rows = Math.round( val.length / 25 );
 	    }
 	    if( rows < 2 ) { rows = 2; }
-	    $( '#' + div_id ).empty().append( '<textarea cols="35" rows="' + rows + '" class="in_edit_same" id="' + txt_id + '">' + val + '</textarea><BR>' +
+	    var w = $( '#' + div_id ).width();
+	    var h = $( '#' + div_id ).height();
+	    $( '#' + div_id ).empty().append( '<textarea STYLE="width:' + w + 'px;' + 
+					      'height:' + h + 'px;" class="in_edit_same" id="' + txt_id + '">' + val + '</textarea><BR>' +
 					'<button class="cancel" type="button" id="' + canc_id + '">cancel</button> ' +
 					'<button class="go" type="button" id="' + go_id + '">Go</button> ' );
 	    $( '#' + txt_id ).keyup( function(e) {
@@ -443,8 +446,8 @@ $.yote.util = {
     login_control:function( args ) {
 	var lc = {
 	    attachpoint      : args[ 'attachpoint' ],
-	    msg_function     : args[ 'msg_function' ]        || function(m,c){},
-	    log_in_status    : args[ 'log_in_status' ]       || '#logged_in_status',
+	    msg_function     : args[ 'msg_function' ]              || function(m,c){ c ? $( '#login_msg' ).removeClass().addClass( c ) : '';$( '#login_msg' ).empty().append( m ); },
+	    log_in_status    : args[ 'log_in_status_attachpoint' ] || '#logged_in_status',
 	    after_login_fun  : args[ 'after_login_function' ],
 	    after_logout_fun : args[ 'after_logout_function' ],
 
@@ -456,9 +459,9 @@ $.yote.util = {
 		);
 		$( '#logout' ).click( function() {
 		    thislc.msg_function( 'logged out' );
-		    $.yote.logout();
 		    $( thislc.log_in_status ).empty();
 		    thislc.on_logout_fun();
+		    $.yote.logout();
 		    thislc.after_logout_fun();
 		} );
 	    }, //on_login
@@ -468,6 +471,7 @@ $.yote.util = {
 		thislc.msg_function('');
 		$( thislc.attachpoint ).empty().append(
 		    '<div class="panel core" id="create_acct_div">' +
+			'<DIV id="login_msg"></DIV>' + 
 			'<P><input type="text" id="username" placeholder="Name" size="6">' +
 			'<input type="email" placeholder="Email (optional)" id="em" size="8">' +
 			'<input type="password" placeholder="Password" id="pw" size="6">' +
@@ -513,6 +517,7 @@ $.yote.util = {
 		var thislc = this;
 		$( thislc.attachpoint ).empty().append(
 		    '<div class="panel core" id="create_acct_div">' +
+			'<DIV id="login_msg"></DIV>' + 
 			'Log In' +
 			'<input type="text" id="username" placeholder="Name" size="6">' +
 			'<input type="password" placeholder="Password" id="pw" size="6"> <BUTTON type="BUTTON" id="log_in_b">Log In</BUTTON></P> ' +
@@ -548,6 +553,14 @@ $.yote.util = {
 	};
 	lc.on_logout_fun = args[ 'on_logout_function' ] || lc.make_login;
 	lc.on_login_fun = args[ 'on_login_fun' ]  || lc.on_login;
+
+	if( $.yote.is_logged_in() ) {
+	    lc.on_login_fun();
+	    lc.after_login_fun();
+	} else {
+	    lc.on_logout_fun();
+	    lc.after_logout_fun();
+	}
 	return lc
     }, //login_control
 
@@ -645,8 +658,8 @@ $.yote.util = {
 
 		var items = paginate_type == 'hash' ?
 		    item.paginate_hash( [ list_name, plimit + 1, me.start ] ) :
-		    paginate_order == 'forward' ? item.paginate( [ list_name, plimit + 1, me.start ] ) :
-		    item.paginate_rev( [ list_name, plimit + 1, me.start ] );
+		    paginate_order == 'forward' ? item.paginate_list( [ list_name, plimit + 1, me.start ] ) :
+		    item.paginate_list_rev( [ list_name, plimit + 1, me.start ] );
 
 		var max = items.length() > plimit ? plimit : items.length();
 
